@@ -1411,7 +1411,7 @@ void File_Mpeg4v::user_data_start()
                     {
                         Date[4]=__T('-');
                         Date[7]=__T('-');
-                        Library_Date=__T("UTC ")+Date;
+                        Library_Date=Date+__T(" UTC");
                     }
                 }
             }
